"""This module is a Robot service for reporting results to Report Portal."""

#  Copyright (c) 2023 EPAM Systems
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License

from dateutil.parser import parse
import logging

from reportportal_client.logs.log_manager import MAX_LOG_BATCH_PAYLOAD_SIZE
from reportportal_client.helpers import (
    dict_to_payload,
    get_launch_sys_attrs,
    get_package_version,
    timestamp
)
from reportportal_client.client import RPClient

from .exception import RobotServiceException
from .static import LOG_LEVEL_MAPPING, STATUS_MAPPING

logger = logging.getLogger(__name__)

TOP_LEVEL_ITEMS = {'BEFORE_SUITE', 'AFTER_SUITE'}


def to_epoch(date):
    """Convert Robot Framework timestamp to UTC timestamp."""
    if not date:
        return None
    try:
        parsed_date = parse(date)
    except ValueError:
        return None
    if hasattr(parsed_date, 'timestamp'):
        epoch_time = parsed_date.timestamp()
    else:
        epoch_time = \
            float(parsed_date.strftime('%s')) + parsed_date.microsecond / 1e6
    return str(int(epoch_time * 1000))


class RobotService(object):
    """Class represents service that sends Robot items to Report Portal."""

    def __init__(self):
        """Initialize service attributes."""
        self.agent_name = 'robotframework-reportportal'
        self.agent_version = get_package_version(self.agent_name)
        self.rp = None

    def _get_launch_attributes(self, cmd_attrs):
        """Generate launch attributes including both system and user ones.

        :param list cmd_attrs: List for attributes from the command line
        """
        attributes = cmd_attrs or []
        system_attributes = get_launch_sys_attrs()
        system_attributes['agent'] = (
            '{}|{}'.format(self.agent_name, self.agent_version))
        return attributes + dict_to_payload(system_attributes)

    def init_service(self, endpoint, project, uuid, log_batch_size, pool_size,
                     skipped_issue=True, verify_ssl=True,
                     log_batch_payload_size=MAX_LOG_BATCH_PAYLOAD_SIZE,
                     launch_id=None):
        """Initialize common Report Portal client.

        :param endpoint:               Report Portal API endpoint
        :param project:                Report Portal project
        :param uuid:                   API token
        :param log_batch_size:         Number of logs to be sent within one
                                       batch
        :param pool_size:              HTTPAdapter max pool size
        :param skipped_issue:          Mark skipped test items with
                                       'To Investigate', default value 'True'
        :param verify_ssl:             Disable SSL verification.
        :param log_batch_payload_size: Maximum size of logs to be sent within
                                       one batch
        :param launch_id:              a launch id to use instead of starting
                                       own one
        """
        if self.rp is None:
            logger.debug(
                'ReportPortal - Init service: '
                'endpoint={0}, project={1}, uuid={2}'
                .format(endpoint, project, uuid))
            self.rp = RPClient(
                endpoint=endpoint,
                project=project,
                token=uuid,
                is_skipped_an_issue=skipped_issue,
                log_batch_size=log_batch_size,
                retries=True,
                verify_ssl=verify_ssl,
                max_pool_size=pool_size,
                log_batch_payload_size=log_batch_payload_size,
                launch_id=launch_id
            )
            self.rp.start()
        else:
            raise RobotServiceException(
                'RobotFrameworkService is already initialized.')

    def terminate_service(self):
        """Terminate common reportportal client."""
        if self.rp is not None:
            self.rp.terminate()

    def start_launch(self, launch, mode=None, rerun=False, rerun_of=None,
                     ts=None):
        """Call start_launch method of the common client.

        :param launch:         Instance of the Launch class
        :param mode:           Launch mode
        :param rerun:          Rerun mode. Allowable values 'True' of 'False'
        :param rerun_of:       Rerun mode. Specifies launch to be re-runned.
                               Should be used with the 'rerun' option.
        :param ts:             Start time
        :return:               launch UUID
        """
        sl_pt = {
            'attributes': self._get_launch_attributes(launch.attributes),
            'description': launch.doc,
            'name': launch.name,
            'mode': mode,
            'rerun': rerun,
            'rerun_of': rerun_of,
            'start_time': ts or to_epoch(launch.start_time) or timestamp()
        }
        logger.debug(
            'ReportPortal - Start launch: request_body={0}'.format(sl_pt))
        return self.rp.start_launch(**sl_pt)

    def finish_launch(self, launch, ts=None):
        """Finish started launch.

        :param launch: Launch name
        :param ts:     End time
        """
        fl_rq = {
            'end_time': ts or to_epoch(launch.end_time) or timestamp(),
            'status': STATUS_MAPPING[launch.status]
        }
        logger.debug(
            'ReportPortal - Finish launch: request_body={0}'.format(fl_rq))
        self.rp.finish_launch(**fl_rq)

    def start_suite(self, suite, ts=None):
        """Call start_test method of the common client.

        :param suite: model.Suite object
        :param ts:    Start time
        :return:      Suite UUID
        """
        start_rq = {
            'attributes': None,
            'description': suite.doc,
            'item_type': suite.type,
            'name': suite.name,
            'parent_item_id': suite.rp_parent_item_id,
            'start_time': ts or to_epoch(suite.start_time) or timestamp()
        }
        logger.debug(
            'ReportPortal - Start suite: request_body={0}'.format(start_rq))
        return self.rp.start_test_item(**start_rq)

    def finish_suite(self, suite, issue=None, ts=None):
        """Finish started suite.

        :param suite: Instance of the started suite item
        :param issue: Corresponding issue if it exists
        :param ts:    End time
        """
        fta_rq = {
            'end_time': ts or to_epoch(suite.end_time) or timestamp(),
            'issue': issue,
            'item_id': suite.rp_item_id,
            'status': STATUS_MAPPING[suite.status]
        }
        logger.debug(
            'ReportPortal - Finish suite: request_body={0}'.format(fta_rq))
        self.rp.finish_test_item(**fta_rq)

    def start_test(self, test, ts=None):
        """Call start_test method of the common client.

        :param test: model.Test object
        :param ts:   Start time
        """
        # Item type should be sent as "STEP" until we upgrade to RPv6.
        # Details at:
        # https://github.com/reportportal/agent-Python-RobotFramework/issues/56
        start_rq = {
            'attributes': test.attributes,
            'code_ref': test.code_ref,
            'description': test.doc,
            'item_type': 'STEP',
            'name': test.name,
            'parent_item_id': test.rp_parent_item_id,
            'start_time': ts or to_epoch(test.start_time) or timestamp(),
            'test_case_id': test.test_case_id
        }
        logger.debug(
            'ReportPortal - Start test: request_body={0}'.format(start_rq))
        return self.rp.start_test_item(**start_rq)

    def finish_test(self, test, issue=None, ts=None):
        """Finish started test case.

        :param test:  Instance of started test item
        :param issue: Corresponding issue if it exists
        :param ts:    End time
        """
        fta_rq = {
            'attributes': test.attributes,
            'end_time': ts or to_epoch(test.end_time) or timestamp(),
            'issue': issue,
            'item_id': test.rp_item_id,
            'status': STATUS_MAPPING[test.status]
        }
        logger.debug(
            'ReportPortal - Finish test: request_body={0}'.format(fta_rq))
        self.rp.finish_test_item(**fta_rq)

    def start_keyword(self, keyword, ts=None):
        """Call start_test method of the common client.

        :param keyword: model.Keyword object
        :param ts:      Start time
        """
        start_rq = {
            'description': keyword.doc,
            'has_stats': keyword.get_type() in TOP_LEVEL_ITEMS,
            'item_type': keyword.get_type(),
            'name': keyword.get_name(),
            'parent_item_id': keyword.rp_parent_item_id,
            'start_time': ts or to_epoch(keyword.start_time) or timestamp()
        }
        logger.debug(
            'ReportPortal - Start keyword: request_body={0}'.format(start_rq))
        return self.rp.start_test_item(**start_rq)

    def finish_keyword(self, keyword, issue=None, ts=None):
        """Finish started keyword item.

        :param keyword: Instance of started keyword item
        :param issue:   Corresponding issue if it exists
        :param ts:      End time
        """
        fta_rq = {
            'end_time': ts or to_epoch(keyword.end_time) or timestamp(),
            'issue': issue,
            'item_id': keyword.rp_item_id,
            'status': STATUS_MAPPING[keyword.status]
        }
        logger.debug(
            'ReportPortal - Finish keyword: request_body={0}'.format(fta_rq))
        self.rp.finish_test_item(**fta_rq)

    def log(self, message, ts=None):
        """Send log message to Report Portal.

        :param message: model.LogMessage object
        :param ts:      Timestamp
        """
        sl_rq = {
            'attachment': message.attachment,
            'item_id': message.item_id,
            'level': LOG_LEVEL_MAPPING.get(message.level, 'INFO'),
            'message': message.message,
            'time': ts or timestamp()
        }
        self.rp.log(**sl_rq)



#  Copyright 2022 EPAM Systems
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Robot Framework test report sender to Report Portal.

This tool replays a Robot Framework test session using the output XML file
with ReportPortal's listener agent injected without actually doing the test.
This will allow all the test results to be sent to ReportPortal, after the
test is run. By doing this, we will not disturb the timing when the test is
run. It also has the benefit of using the test result generated after running
multiple parallel robot testing (eg. by using pabot).

Command-line usage:

    post_report --variable RP_UUID:"your_user_uuid"
                --variable RP_ENDPOINT:"your_reportportal_url"
                --variable RP_PROJECT:"reportportal_project_name"
                [--variable RP_LAUNCH:"launch_name"]
                [--variable RP_LAUNCH_DOC:"some_documentation_for_launch"]
                [--variable RP_LAUNCH_ATTRIBUTES:"RF tag_name:tag_value"]
                [--variable RP_TEST_ATTRIBUTES:"Long"]
                [--variable RP_LOG_BATCH_SIZE:"10"]
                [--variable RP_MAX_POOL_SIZE:"50"]
                [--variable RP_MODE:"DEBUG"]
                [--loglevel CRITICAL|ERROR|WARNING|INFO|DEBUG]
                [output.xml]

This script needs to be run within the same directory as the report xml file.
Attachments mentioned in the log messages will be referred relative to
current dir.
"""

import getopt
import logging
import sys

from robot.api import ExecutionResult

from robotframework_reportportal.result_visitor import RobotResultsVisitor
from robotframework_reportportal.time_visitor import TimeVisitor, corrections
# noinspection PyUnresolvedReferences
from robotframework_reportportal.variables import _variables


def process(infile="output.xml"):
    test_run = ExecutionResult(infile)
    test_run.visit(TimeVisitor())
    if corrections:
        logging.warning("{0} is missing some of its starttime/endtime. "
                        "This might cause inconsistencies with your "
                        "duration report.".format(infile))
    test_run.visit(RobotResultsVisitor())


def main():
    argument_list = sys.argv[1:]
    short_options = "hv:"
    long_options = ["help", "variable=", "loglevel="]
    try:
        arguments, values = getopt.getopt(argument_list, short_options,
                                          long_options)
    except getopt.error:
        sys.exit(1)

    for current_argument, current_value in arguments:
        if current_argument in ("-h", "--help"):
            print(__doc__)
            sys.exit(0)
        elif current_argument in ("-v", "--variable"):
            k, v = str(current_value).split(":", 1)
            _variables[k] = v
        elif current_argument == "--loglevel":
            numeric_level = getattr(logging, current_value.upper(), None)
            logging.basicConfig(level=numeric_level)

    try:
        process(*values)
    except TypeError:
        print(__doc__)
        sys.exit(1)
    sys.exit(0)


if __name__ == "__main__":
    main()



"""This module includes Robot Framework listener interfaces."""

#  Copyright 2023 EPAM Systems
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import logging
import os
from mimetypes import guess_type

from reportportal_client.helpers import gen_attributes

from .exception import RobotServiceException
from .model import Keyword, Launch, Test, LogMessage, Suite
from .service import RobotService
from .static import MAIN_SUITE_ID, PABOT_WIHOUT_LAUNCH_ID_MSG
from .variables import Variables

logger = logging.getLogger(__name__)


class listener(object):
    """Robot Framework listener interface for reporting to Report Portal."""

    ROBOT_LISTENER_API_VERSION = 2

    def __init__(self):
        """Initialize listener attributes."""
        self._items = []
        self._service = None
        self._variables = None

    def _build_msg_struct(self, message):
        """Check if the given message comes from our custom logger or not.

        :param message: Message passed by the Robot Framework
        """
        if isinstance(message['message'], LogMessage):
            msg = message['message']
        else:
            msg = LogMessage(message['message'])
            msg.level = message['level']
        if not msg.launch_log:
            msg.item_id = getattr(self.current_item, 'rp_item_id', None)
        return msg

    def _finish_current_item(self):
        """Remove the last item from the self._items list."""
        return self._items.pop()

    @property
    def current_item(self):
        """Get the last item from the self._items list."""
        if self._items:
            return self._items[-1]

    def log_message(self, message):
        """Send log message to the Report Portal.

        :param message: Message passed by the Robot Framework
        """
        msg = self._build_msg_struct(message)
        logger.debug('ReportPortal - Log Message: {0}'.format(message))
        self.service.log(message=msg)

    def log_message_with_image(self, msg, image):
        """Send log message to the Report Portal.

        :param msg:   Message passed by the Robot Framework
        :param image: Path to image
        """
        mes = self._build_msg_struct(msg)
        with open(image, 'rb') as fh:
            mes.attachment = {
                'name': os.path.basename(image),
                'data': fh.read(),
                'mime': guess_type(image)[0] or 'application/octet-stream'
            }
        logger.debug('ReportPortal - Log Message with Image: {0} {1}'
                     .format(mes, image))
        self.service.log(message=mes)

    @property
    def parent_id(self):
        """Get rp_item_id attribute of the current item."""
        return getattr(self.current_item, 'rp_item_id', None)

    @property
    def service(self):
        """Initialize instance of the RobotService."""
        if self._service is None:
            self._service = RobotService()
            self._service.init_service(
                endpoint=self.variables.endpoint,
                project=self.variables.project,
                uuid=self.variables.uuid,
                log_batch_size=self.variables.log_batch_size,
                pool_size=self.variables.pool_size,
                skipped_issue=self.variables.skipped_issue,
                verify_ssl=self.variables.verify_ssl,
                log_batch_payload_size=self.variables.log_batch_payload_size,
                launch_id=self.variables.launch_id,
            )
        return self._service

    @property
    def variables(self):
        """Get instance of the variables.Variables class."""
        if not self._variables:
            self._variables = Variables()
        return self._variables

    def start_launch(self, attributes, ts=None):
        """Start a new launch at the Report Portal.

        :param attributes: Dictionary passed by the Robot Framework
        :param ts:         Timestamp(used by the ResultVisitor)
        """
        launch = Launch(self.variables.launch_name, attributes)
        launch.attributes = gen_attributes(self.variables.launch_attributes)
        launch.doc = self.variables.launch_doc or launch.doc
        if not self.variables.launch_id:
            if self.variables.pabot_used:
                raise RobotServiceException(PABOT_WIHOUT_LAUNCH_ID_MSG)
            logger.debug('ReportPortal - Start Launch: {0}'.format(
                launch.attributes))
            self.service.start_launch(
                launch=launch,
                mode=self.variables.mode,
                ts=ts,
                rerun=self.variables.rerun,
                rerun_of=self.variables.rerun_of)
        else:
            self.service.rp.launch_id = self.variables.launch_id

    def start_suite(self, name, attributes, ts=None):
        """Start a new test suite at the Report Portal.

        :param name:       Test suite name
        :param attributes: Dictionary passed by the Robot Framework
        :param ts:         Timestamp(used by the ResultVisitor)
        """
        if attributes['id'] == MAIN_SUITE_ID:
            self.start_launch(attributes, ts)
            if self.variables.pabot_used:
                name += '.{0}'.format(self.variables.pabot_pool_id)
            logger.debug(
                'ReportPortal - Create global Suite: {0}'
                .format(attributes))
            suite = Suite(name, attributes)
            suite.rp_item_id = self.service.start_suite(suite=suite, ts=ts)
            self._items.append(suite)
        else:
            logger.debug('ReportPortal - Start Suite: {0}'.format(attributes))
            suite = Suite(name, attributes)
            suite.rp_parent_item_id = self.parent_id
            suite.rp_item_id = self.service.start_suite(suite=suite, ts=ts)
            self._items.append(suite)

    def end_suite(self, _, attributes, ts=None):
        """Finish started test suite at the Report Portal.

        :param attributes: Dictionary passed by the Robot Framework
        :param ts:         Timestamp(used by the ResultVisitor)
        """
        if attributes['id'] == MAIN_SUITE_ID:
            suite = self._finish_current_item().update(attributes)
            logger.debug('ReportPortal - End Suite: {0}'
                         .format(suite.attributes))
            self.service.finish_suite(suite=suite, ts=ts)
            launch = Launch(self.variables.launch_name, attributes)
            logger.debug(
                msg='ReportPortal - End Launch: {0}'.format(attributes))
            self.service.finish_launch(launch=launch, ts=ts)
        else:
            suite = self._finish_current_item().update(attributes)
            logger.debug(
                'ReportPortal - End Suite: {0}'.format(suite.attributes))
            self.service.finish_suite(suite=suite, ts=ts)

    def start_test(self, name, attributes, ts=None):
        """Start a new test case at the Report Portal.

        :param name:       Test case name
        :param attributes: Dictionary passed by the Robot Framework
        :param ts:         Timestamp(used by the ResultVisitor)
        """
        if 'source' not in attributes:
            # no 'source' parameter at this level for Robot versions < 4
            attributes = attributes.copy()
            attributes['source'] = getattr(self.current_item, 'source', None)
        test = Test(name=name, attributes=attributes)
        logger.debug('ReportPortal - Start Test: {0}'.format(attributes))
        test.attributes = gen_attributes(
            self.variables.test_attributes + test.tags)
        test.rp_parent_item_id = self.parent_id
        test.rp_item_id = self.service.start_test(test=test, ts=ts)
        self._items.append(test)

    def end_test(self, _, attributes, ts=None):
        """Finish started test case at the Report Portal.

        :param attributes: Dictionary passed by the Robot Framework
        :param ts:         Timestamp(used by the ResultVisitor)
        """
        test = self.current_item.update(attributes)
        test.attributes = gen_attributes(
            self.variables.test_attributes + test.tags)
        if not test.critical and test.status == 'FAIL':
            test.status = 'SKIP'
        if test.message:
            self.log_message({'message': test.message, 'level': 'DEBUG'})
        logger.debug('ReportPortal - End Test: {0}'.format(test.attributes))
        self._finish_current_item()
        self.service.finish_test(test=test, ts=ts)

    def start_keyword(self, name, attributes, ts=None):
        """Start a new keyword(test step) at the Report Portal.

        :param name:       Keyword name
        :param attributes: Dictionary passed by the Robot Framework
        :param ts:         Timestamp(used by the ResultVisitor)
        """
        kwd = Keyword(name=name, parent_type=self.current_item.type,
                      attributes=attributes)
        kwd.rp_parent_item_id = self.parent_id
        logger.debug('ReportPortal - Start Keyword: {0}'.format(attributes))
        kwd.rp_item_id = self.service.start_keyword(keyword=kwd, ts=ts)
        self._items.append(kwd)

    def end_keyword(self, _, attributes, ts=None):
        """Finish started keyword at the Report Portal.

        :param attributes: Dictionary passed by the Robot Framework
        :param ts:         Timestamp(used by the ResultVisitor)
        """
        kwd = self._finish_current_item().update(attributes)
        logger.debug('ReportPortal - End Keyword: {0}'.format(kwd.attributes))
        self.service.finish_keyword(keyword=kwd, ts=ts)

    def log_file(self, log_path):
        """Attach HTML log file created by Robot Framework to RP launch.

        :param log_path: Path to the log file
        """
        if self.variables.attach_log:
            message = {'message': 'Execution log', 'level': 'INFO'}
            self.log_message_with_image(message, log_path)

    def report_file(self, report_path):
        """Attach HTML report created by Robot Framework to RP launch.

        :param report_path: Path to the report file
        """
        if self.variables.attach_report:
            message = {'message': 'Execution report', 'level': 'INFO'}
            self.log_message_with_image(message, report_path)

    def xunit_file(self, xunit_path):
        """Attach XUnit file created by Robot Framework to RP launch.

        :param xunit_path: Path to the XUnit file
        """
        if self.variables.attach_xunit:
            message = {'message': 'XUnit result file', 'level': 'INFO'}
            self.log_message_with_image(message, xunit_path)

    def close(self):
        """Call service terminate when the whole test execution is done."""
        self.service.terminate_service()



"""This module contains model that stores Robot Framework variables."""

#  Copyright (c) 2023 EPAM Systems
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License

from distutils.util import strtobool
from os import path

from reportportal_client.logs.log_manager import MAX_LOG_BATCH_PAYLOAD_SIZE
from robot.libraries.BuiltIn import BuiltIn, RobotNotRunningError

from .exception import RobotServiceException

# This is a storage for the result visitor
_variables = {}


def get_variable(name, default=None):
    """Get Robot Framework variable.

    :param name:    Name of the variable
    :param default: Default value
    :return:        The value of the variable, otherwise default value
    """
    try:
        return BuiltIn().get_variable_value("${" + name + "}", default=default)
    except RobotNotRunningError:
        return _variables.get(name, default)


class Variables(object):
    """This class stores Robot Framework variables related to Report Portal."""

    def __init__(self):
        """Initialize instance attributes."""
        self._endpoint = None
        self._launch_name = None
        self._pabot_pool_id = None
        self._pabot_used = None
        self._project = None
        self._uuid = None
        self.attach_log = bool(strtobool(get_variable(
            'RP_ATTACH_LOG', default='False')))
        self.attach_report = bool(strtobool(get_variable(
            'RP_ATTACH_REPORT', default='False')))
        self.attach_xunit = bool(strtobool(get_variable(
            'RP_ATTACH_XUNIT', default='False')))
        self.launch_attributes = get_variable(
            'RP_LAUNCH_ATTRIBUTES', default='').split()
        self.launch_id = get_variable('RP_LAUNCH_UUID')
        self.launch_doc = get_variable('RP_LAUNCH_DOC')
        self.log_batch_size = int(get_variable(
            'RP_LOG_BATCH_SIZE', default='20'))
        self.mode = get_variable('RP_MODE')
        self.pool_size = int(get_variable('RP_MAX_POOL_SIZE', default='50'))
        self.rerun = bool(strtobool(get_variable(
            'RP_RERUN', default='False')))
        self.rerun_of = get_variable('RP_RERUN_OF', default=None)
        self.skipped_issue = bool(strtobool(get_variable(
            'RP_SKIPPED_ISSUE', default='True')))
        self.test_attributes = get_variable(
            'RP_TEST_ATTRIBUTES', default='').split()
        self.log_batch_payload_size = int(get_variable(
            "RP_LOG_BATCH_PAYLOAD_SIZE",
            default=str(MAX_LOG_BATCH_PAYLOAD_SIZE)))

    @property
    def endpoint(self):
        """Get Report Portal API endpoint.

        :raises: RobotServiceException if it is None
        :return: Report Portal API endpoint
        """
        self._endpoint = self._endpoint or get_variable('RP_ENDPOINT')
        if self._endpoint is None:
            raise RobotServiceException(
                'Missing parameter RP_ENDPOINT for robot run\n'
                'You should pass -v RP_ENDPOINT:<endpoint_value>')
        return self._endpoint

    @property
    def launch_name(self):
        """Get Report Portal launch name.

        :raises: RobotServiceException if it is None
        :return: Report Portal launch name
        """
        self._launch_name = self._launch_name or get_variable('RP_LAUNCH')
        if self._launch_name is None:
            raise RobotServiceException(
                'Missing parameter RP_LAUNCH for robot run\n'
                'You should pass -v RP_LAUNCH:<launch_name_value>')
        return self._launch_name

    @property
    def pabot_pool_id(self):
        """Get pool id for the current Robot Framework executor.

        :return: Pool id for the current Robot Framework executor
        """
        if not self._pabot_pool_id:
            self._pabot_pool_id = get_variable(name='PABOTEXECUTIONPOOLID')
        return self._pabot_pool_id

    @property
    def pabot_used(self):
        """Get status of using pabot in test execution.

        :return: Cached value of the Pabotlib URI
        """
        if not self._pabot_used:
            self._pabot_used = get_variable(name='PABOTLIBURI')
        return self._pabot_used

    @property
    def project(self):
        """Get Report Portal project name.

        :raises: RobotServiceException if it is None
        :return: Report Portal project name
        """
        self._project = self._project or get_variable('RP_PROJECT')
        if self._project is None:
            raise RobotServiceException(
                'Missing parameter RP_PROJECT for robot run\n'
                'You should pass -v RP_PROJECT:<project_name_value>')
        return self._project

    @property
    def uuid(self):
        """Get Report Portal API token UUID.

        :raises: RobotServiceException if it is None
        :return: Report Portal token UUID
        """
        self._uuid = self._uuid or get_variable('RP_UUID')
        if self._uuid is None:
            raise RobotServiceException(
                'Missing parameter RP_UUID for robot run\n'
                'You should pass -v RP_UUID:<uuid_value>')
        return self._uuid

    @property
    def verify_ssl(self):
        """Get value of the verify_ssl parameter for the client."""
        verify_ssl = get_variable('RP_VERIFY_SSL', default='True')
        if path.exists(verify_ssl):
            return verify_ssl
        return bool(strtobool(verify_ssl))



#  Copyright 2022 EPAM Systems
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import logging

from robot.api import ResultVisitor

_stack = []
corrections = {}


class TimeVisitor(ResultVisitor):

    @staticmethod
    def _correct_starts(o, node_class):
        """
        starttime wants to be the oldest start time of its children.
        only correcting null starttime.
        """
        if o.starttime:
            corrected = False
            for parent_id in _stack:
                if corrections[parent_id][0] is None or \
                        corrections[parent_id][0] > o.starttime:
                    corrections[parent_id][0] = o.starttime
                    corrected = True
            if corrected:
                logging.debug(
                    "Correcting parents' starttime to {0} based on {2}={1}"
                    .format(o.starttime, o.id, node_class))
        else:
            _stack.append(o.id)
            corrections[o.id] = [None, None]

    @staticmethod
    def _correct_ends(o, node_class):
        """
        endtime wants to be the newest end time of its children.
        only correcting null endtime.
        """
        if o.endtime:
            corrected = False
            for parent_id in _stack:
                if corrections[parent_id][1] is None or \
                        corrections[parent_id][1] < o.endtime:
                    corrections[parent_id][1] = o.endtime
                    corrected = True
            if corrected:
                logging.debug(
                    "Correcting parents' endtime to {0} based on {2}={1}"
                    .format(o.endtime, o.id, node_class))
        if _stack and o.id == _stack[-1]:
            _stack.pop()

    def start_suite(self, suite):
        self._correct_starts(suite, "suite")

    def end_suite(self, suite):
        self._correct_ends(suite, "suite")

    def start_test(self, test):
        self._correct_starts(test, "test")

    def end_test(self, test):
        self._correct_ends(test, "test")

    def start_keyword(self, keyword):
        self._correct_starts(keyword, "kw")

    def end_keyword(self, keyword):
        self._correct_ends(keyword, "kw")



"""This module contains models representing Robot Framework test items."""

#  Copyright (c) 2023 EPAM Systems
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License

import os

from reportportal_client.helpers import convert_string
from six import text_type


class Suite(object):
    """Class represents Robot Framework test suite."""

    def __init__(self, name, attributes):
        """Initialize required attributes.

        :param name:       Suite name
        :param attributes: Suite attributes passed through the listener
        """
        self.attributes = attributes
        self.doc = attributes['doc']
        self.end_time = attributes.get('endtime', '')
        self.longname = attributes['longname']
        self.message = attributes.get('message')
        self.metadata = attributes['metadata']
        self.name = name
        self.robot_id = attributes['id']
        self.rp_item_id = None
        self.rp_parent_item_id = None
        self.start_time = attributes.get('starttime')
        self.statistics = attributes.get('statistics')
        self.status = attributes.get('status')
        self.suites = attributes['suites']
        self.tests = attributes['tests']
        self.total_tests = attributes['totaltests']
        self.type = 'SUITE'

    @property
    def source(self):
        """Return the test case source file path."""
        if self.attributes.get('source') is not None:
            return os.path.relpath(self.attributes['source'], os.getcwd())

    def update(self, attributes):
        """Update suite attributes on suite finish.

        :param attributes: Suite attributes passed through the listener
        """
        self.end_time = attributes.get('endtime', '')
        self.message = attributes.get('message')
        self.status = attributes.get('status')
        self.statistics = attributes.get('statistics')
        return self


class Launch(Suite):
    """Class represents Robot Framework test suite."""

    def __init__(self, name, attributes):
        """Initialize required attributes.

        :param name:       Launch name
        :param attributes: Suite attributes passed through the listener
        """
        # noinspection PySuperArguments
        super(Launch, self).__init__(name, attributes)
        self.type = 'LAUNCH'


class Test(object):
    """Class represents Robot Framework test case."""

    def __init__(self, name, attributes):
        """Initialize required attributes.

        :param name:       Name of the test
        :param attributes: Test attributes passed through the listener
        """
        # for backward compatibility with Robot < 4.0 mark every test case
        # as critical if not set
        self._critical = attributes.get('critical', 'yes')
        self._tags = attributes['tags']
        self._attributes = attributes
        self.doc = attributes['doc']
        self.end_time = attributes.get('endtime', '')
        self.longname = attributes['longname']
        self.message = attributes.get('message')
        self.name = name
        self.robot_id = attributes['id']
        self.rp_item_id = None
        self.rp_parent_item_id = None
        self.start_time = attributes['starttime']
        self.status = attributes.get('status')
        self.template = attributes['template']
        self.type = 'TEST'

    @property
    def critical(self):
        """Form unique value for RF 4.0+ and older versions."""
        return self._critical in ('yes', True)

    @property
    def tags(self):
        """Get list of test tags excluding test_case_id."""
        return [
            tag for tag in self._tags if not tag.startswith('test_case_id')]

    @property
    def source(self):
        """Return the test case source file path."""
        if self._attributes['source'] is not None:
            return os.path.relpath(self._attributes['source'], os.getcwd())

    @property
    def code_ref(self):
        """Return the test case code reference.

        The result line should be exactly how it appears in '.robot' file.
        """
        line_number = self._attributes.get("lineno")
        if line_number is not None:
            return '{0}:{1}'.format(self.source, line_number)
        return '{0}:{1}'.format(self.source, self.name)

    @property
    def test_case_id(self):
        """Get test case ID through the tags."""
        # use test case id from tags if specified
        for tag in self._tags:
            if tag.startswith('test_case_id:'):
                return tag.split(':')[1]
        # generate it if not
        return '{0}:{1}'.format(self.source, self.name)

    def update(self, attributes):
        """Update test attributes on test finish.

        :param attributes: Suite attributes passed through the listener
        """
        self._tags = attributes.get('tags', self._tags)
        self.end_time = attributes.get('endtime', '')
        self.message = attributes.get('message')
        self.status = attributes.get('status')
        return self


class Keyword(object):
    """Class represents Robot Framework keyword."""

    def __init__(self, name, attributes, parent_type=None):
        """Initialize required attributes.

        :param name:        Name of the keyword
        :param attributes:  Keyword attributes passed through the listener
        :param parent_type: Type of the parent test item
        """
        self.attributes = attributes
        self.args = attributes['args']
        self.assign = attributes['assign']
        self.doc = attributes['doc']
        self.end_time = attributes.get('endtime')
        self.keyword_name = attributes['kwname']
        self.keyword_type = attributes['type']
        self.libname = attributes['libname']
        self.name = name
        self.rp_item_id = None
        self.rp_parent_item_id = None
        self.parent_type = parent_type
        self.start_time = attributes['starttime']
        self.status = attributes.get('status')
        self.tags = attributes['tags']
        self.type = 'KEYWORD'

    def get_name(self):
        """Get name of the keyword suitable for Report Portal."""
        assign = convert_string(', '.join(self.assign))
        assignment = '{0} = '.format(assign) if self.assign else ''
        arguments = ', '.join(self.args)
        full_name = '{0}{1} ({2})'.format(
            assignment,
            convert_string(self.name),
            convert_string(arguments)
        )
        return full_name[:256]

    def get_type(self):
        """Get keyword type."""
        if self.keyword_type.lower() in ('setup', 'teardown'):
            if self.parent_type.lower() == 'keyword':
                return 'STEP'
            if self.keyword_type.lower() == 'setup':
                return 'BEFORE_{0}'.format(self.parent_type.upper())
            if self.keyword_type.lower() == 'teardown':
                return 'AFTER_{0}'.format(self.parent_type.upper())
        else:
            return 'STEP'

    def update(self, attributes):
        """Update keyword attributes on keyword finish.

        :param attributes: Suite attributes passed through the listener
        """
        self.end_time = attributes.get('endtime', '')
        self.status = attributes.get('status')
        return self


class LogMessage(text_type):
    """Class represents Robot Framework messages."""

    def __init__(self, message):
        """Initialize required attributes."""
        self.attachment = None
        self.item_id = None
        self.level = 'INFO'
        self.launch_log = False
        self.message = message

    def __repr__(self):
        """Return string representation of the object."""
        return self.message



"""Setup instructions for the package."""

import os
from setuptools import setup, find_packages


__version__ = '5.3.2'


def read_file(fname):
    """Read the given file.

    :param fname: Name of the file to be read
    :return:      Output of the given file
    """
    with open(os.path.join(os.path.dirname(__file__), fname)) as f:
        return f.read()


setup(
    name='robotframework-reportportal',
    packages=find_packages(exclude=('tests', 'tests.*')),
    version=__version__,
    description='Agent for reporting RobotFramework test results to '
                'Report Portal',
    long_description=read_file('README.md'),
    long_description_content_type='text/markdown',
    author_email='support@reportportal.io',
    url='https://github.com/reportportal/agent-Python-RobotFramework',
    download_url=(
        'https://github.com/reportportal/agent-Python-RobotFramework/'
        'tarball/{version}'.format(version=__version__)),
    keywords=['testing', 'reporting', 'robot framework', 'reportportal',
              'agent'],
    classifiers=[
        'Framework :: Robot Framework',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        ],
    install_requires=read_file('requirements.txt').splitlines(),
    entry_points={
        'console_scripts': [
            'post_report=robotframework_reportportal.post_report:main'
        ]
    },
)



#  Copyright 2022 EPAM Systems
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#  https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import re
import string
from datetime import datetime

from robot.api import ResultVisitor
from six.moves.urllib.parse import unquote

from . import listener
from .time_visitor import corrections
# noinspection PyUnresolvedReferences
from .variables import _variables


listener = listener.listener()


def to_timestamp(time_str):
    if time_str:
        dt = datetime.strptime(time_str, '%Y%m%d %H:%M:%S.%f')
        return str(int(dt.timestamp() * 1000))
    return None


class RobotResultsVisitor(ResultVisitor):
    _link_pattern = re.compile("src=[\"\']([^\"\']+)[\"\']")

    def start_result(self, result):
        if "RP_LAUNCH" not in _variables:
            _variables["RP_LAUNCH"] = result.suite.name
        if "RP_LAUNCH_DOC" not in _variables:
            _variables["RP_LAUNCH_DOC"] = result.suite.doc

    def start_suite(self, suite):
        ts = to_timestamp(suite.starttime if suite.id not in corrections else corrections[suite.id][0])
        attrs = {
            'id': suite.id,
            'longname': suite.longname,
            'doc': suite.doc,
            'metadata': suite.metadata,
            'source': suite.source,
            'suites': suite.suites,
            'tests': suite.tests,
            'totaltests': getattr(suite.statistics, 'all', suite.statistics).total,
            'starttime': ts
        }
        listener.start_suite(suite.name, attrs, ts)

    def end_suite(self, suite):
        ts = to_timestamp(suite.endtime if suite.id not in corrections else corrections[suite.id][1])
        attrs = {
            'id': suite.id,
            'longname': suite.longname,
            'doc': suite.doc,
            'metadata': suite.metadata,
            'source': suite.source,
            'suites': suite.suites,
            'tests': suite.tests,
            'totaltests': getattr(suite.statistics, 'all', suite.statistics).total,
            'endtime': ts,
            'elapsedtime': suite.elapsedtime,
            'status': suite.status,
            'statistics': suite.statistics,
            'message': suite.message,
        }
        listener.end_suite(None, attrs, ts)

    def start_test(self, test):
        ts = to_timestamp(test.starttime if test.id not in corrections else corrections[test.id][0])
        attrs = {
            'id': test.id,
            'longname': test.longname,
            # 'originalname': test.originalname,
            'doc': test.doc,
            'tags': list(test.tags),
            # for backward compatibility with Robot < 4.0 mark every test case
            # as critical if not set
            'critical': getattr(test, 'critical', 'yes'),
            'source': test.source,
            'template': '',
            # 'lineno': test.lineno,
            'starttime': ts,
        }
        listener.start_test(test.name, attrs, ts)

    def end_test(self, test):
        ts = to_timestamp(test.endtime if test.id not in corrections else corrections[test.id][1])
        attrs = {
            'id': test.id,
            'longname': test.longname,
            # 'originalname': test.originalname,
            'doc': test.doc,
            'tags': list(test.tags),
            # for backward compatibility with Robot < 4.0 mark every test case
            # as critical if not set
            'critical': getattr(test, 'critical', 'yes'),
            'template': '',
            # 'lineno': test.lineno,
            'endtime': ts,
            'elapsedtime': test.elapsedtime,
            'source': test.source,
            'status': test.status,
            'message': test.message,
        }
        listener.end_test(test.name, attrs, ts)

    def start_keyword(self, kw):
        ts = to_timestamp(kw.starttime if kw.id not in corrections else corrections[kw.id][0])
        attrs = {
            'type': string.capwords(kw.type),
            'kwname': kw.kwname,
            'libname': kw.libname,
            'doc': kw.doc,
            'args': kw.args,
            'assign': kw.assign,
            'tags': kw.tags,
            'starttime': ts,
        }
        listener.start_keyword(kw.name, attrs, ts)

    def end_keyword(self, kw):
        ts = to_timestamp(kw.endtime if kw.id not in corrections else corrections[kw.id][1])
        attrs = {
            'type': string.capwords(kw.type),
            'kwname': kw.kwname,
            'libname': kw.libname,
            'doc': kw.doc,
            'args': kw.args,
            'assign': kw.assign,
            'tags': kw.tags,
            'endtime': ts,
            'elapsedtime': kw.elapsedtime,
            'status': 'PASS' if kw.assign else kw.status,
        }
        listener.end_keyword(kw.name, attrs, ts)

    def start_message(self, msg):
        if msg.message:
            message = {
                'message': msg.message,
                'level': msg.level,
            }
            try:
                m = self.parse_message(message['message'])
                message["message"] = m[0]
                listener.log_message_with_image(message, m[1])
            except (AttributeError, IOError):
                # noinspection PyBroadException
                try:
                    listener.log_message(message)
                except Exception:
                    pass

    def parse_message(self, msg):
        m = self._link_pattern.search(msg)
        return [m.group(), unquote(m.group(1))]
